/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.impl 2.12
import QtQuick.Templates 2.12 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.Slider {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitHandleWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitHandleHeight + topPadding + bottomPadding)

    padding: slider.padding

    hoverEnabled: true

//    property var handleColor: !control.enabled ? slider.disableHandleColor : handleClick ? slider.clickHandleColor :
//                               handleHover ?slider.hoverHandleColor : slider.normalHandleColor
//    property var handleBorderColor: !control.enabled ? slider.disableHandleBorderColor : handleClick ? slider.clickHandleBorderColor :
//                                    handleHover ? slider.hoverHandleBorderColor : slider.normalHandleBorderColor

//    property bool handleClick: control.handle.pressed
//    property bool handleHover: control.handle.hovered
    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    handle: Rectangle {
        id: handleRec
        x: control.leftPadding + (control.horizontal ? control.visualPosition * (control.availableWidth - width) : (control.availableWidth - width) / 2)
        y: control.topPadding + (control.horizontal ? (control.availableHeight - height) / 2 : control.visualPosition * (control.availableHeight - height))
        implicitWidth: slider.handleHeight
        implicitHeight: slider.handleHeight
        radius: width / 2
        color: gradientHandleRec.visible ? "transparent" : getStartColor(!control.enabled ? slider.disableHandleColor : pressed ? slider.clickHandleColor :
               hovered ?slider.hoverHandleColor : slider.normalHandleColor)
        border.width: slider.handleBorderWidth
        border.color: getStartColor(!control.enabled ? slider.disableHandleBorderColor : pressed ? slider.clickHandleBorderColor :
                                    hovered ? slider.hoverHandleBorderColor : slider.normalHandleBorderColor)

        Rectangle{
            id: gradientHandleRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                GradientStop { position: 0.0; color: getStartColor(!control.enabled ? slider.disableHandleColor : pressed ? slider.clickHandleColor :
                                                                   hovered ?slider.hoverHandleColor : slider.normalHandleColor) }

                GradientStop { position: 1.0; color: getEndColor(!control.enabled ? slider.disableHandleColor : pressed ? slider.clickHandleColor :
                                                                 hovered ?slider.hoverHandleColor : slider.normalHandleColor)}
            }
            visible: !parseInterface.isSolidPattern(!control.enabled ? slider.disableHandleColor : pressed ? slider.clickHandleColor :
                                                                                                             hovered ?slider.hoverHandleColor : slider.normalHandleColor)
        }


    }

    background: Item{
        id: item

        implicitWidth: control.horizontal ? (slider.normalWidth + handle.width) : slider.normalHeight
        implicitHeight: control.horizontal ? slider.normalHeight : (slider.normalWidth + handle.height)

        Rectangle {
            width: item.width
            height: item.height
            border.color: getStartColor(slider.focusBorderColor)
            border.width: control.activeFocus ? slider.focusBorderWidth : 0
            scale: control.horizontal && control.mirrored ? -1 : 1
            color: "transparent"

            Rectangle{
                id: unGroove
                x: (control.horizontal ? handle.width / 2 : ((item.width - width) / 2))
                y:(control.horizontal ? ((item.height - height) / 2) : handle.height / 2)
                width: control.horizontal ? (item.width - handle.width) : slider.grooveHeight
                height: control.horizontal ? slider.grooveHeight : (item.height - handle.height)
                radius: control.horizontal ? height/2 : width/2
                border.width: slider.grooveBorderWidth
                border.color: getStartColor(!control.enabled ? slider.disableUnGrooveBorderColor : slider.normalUnGrooveBorderColor)
                color: gradientUNGrooveRec.visible ? "transparent" : getStartColor(!control.enabled ? slider.disableUnGrooveColor : slider.normalUnGrooveColor)

                Rectangle{
                    id: gradientUNGrooveRec
                    anchors.fill: parent
                    radius: parent.radius
                    border.width: parent.border.width
                    border.color: parent.border.color
                    gradient:  Gradient {
                        GradientStop { position: 0.0; color: getStartColor(!control.enabled ? slider.disableUnGrooveColor : slider.normalUnGrooveColor) }

                        GradientStop { position: 1.0; color: getEndColor(!control.enabled ? slider.disableUnGrooveColor : slider.normalUnGrooveColor)}
                    }
                    visible: !parseInterface.isSolidPattern(!control.enabled ? slider.disableUnGrooveColor : slider.normalUnGrooveColor)
                }
            }

            Rectangle{
                id:groove
                x: (control.horizontal ? handle.width / 2  : ((item.width - width) / 2))
                y: (control.horizontal ? ((item.height - height) / 2) : handle.y +  handle.height / 2)
                width: control.horizontal ? control.position * unGroove.width : slider.grooveHeight
                height: control.horizontal ? slider.grooveHeight : control.position * unGroove.height
                radius: control.horizontal ? height/2 : width/2
                border.width: slider.grooveBorderWidth
                border.color: getStartColor(!control.enabled ? slider.disableGrooveBorderColor : slider.normalGrooveBorderColor)
                color: gradientGrooveRec.visible ? "transparent" : getStartColor(!control.enabled ? slider.disableGrooveColor : slider.normalGrooveColor)
                Rectangle{
                    id: gradientGrooveRec
                    anchors.fill: parent
                    radius: parent.radius
                    border.width: parent.border.width
                    border.color: parent.border.color
                    gradient:  Gradient {
                        GradientStop { position: 0.0; color: getStartColor(!control.enabled ? slider.disableGrooveColor : slider.normalGrooveColor) }

                        GradientStop { position: 1.0; color: getEndColor(!control.enabled ? slider.disableGrooveColor : slider.normalGrooveColor)}
                    }
                    visible: !parseInterface.isSolidPattern(!control.enabled ? slider.disableGrooveColor : slider.normalGrooveColor)
                }
            }

//            Rectangle {
//                id: handle
//                x: control.horizontal ? (control.position * control.width - width/2) : ((item.width - width) / 2)
//                y: control.horizontal ? ((item.height - height) / 2) : (control.position * parent.height - height/2)
//                width: slider.grooveHeight
//                height: slider.grooveHeight

//                radius: slider.grooveHeight / 2
//                color: !control.enabled ? slider.disableHandleColor : slider.normalHandleColor
//                border.width: slider.handleBorderWidth
//                border.color: !control.enabled ? slider.disableHandleBorderColor : slider.normalHandleBorderColor
//            }
        }
    }

    StylePrivate.UKUISlider{
        id: slider
    }

    StylePrivate.APPParameter{
        id: app
    }
}
