/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.12
import QtQuick.Templates 2.12 as T
import QtQuick.Controls 2.12
import QtQuick.Controls.impl 2.12
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.Switch {
    id: control

    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            implicitContentWidth + leftPadding + rightPadding)
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             implicitContentHeight + topPadding + bottomPadding,
                             implicitIndicatorHeight + topPadding + bottomPadding)

    padding: ukuiSwitch.padding
    spacing: 6

    StylePrivate.UKUISwitch{
    id: ukuiSwitch
    }
    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    indicator: Rectangle {
        implicitWidth: ukuiSwitch.recWidth
        implicitHeight: ukuiSwitch.recHeight

        x: control.text ? (control.mirrored ? control.width - width - control.rightPadding : control.leftPadding) : control.leftPadding + (control.availableWidth - width) / 2
        y: control.topPadding + (control.availableHeight - height) / 2

        radius: implicitHeight/2
//        leftPadding: 0
//        rightPadding: 0
//        padding: ukuiSwitch.padding
        color: getStartColor(control.checked ? (!control.enabled ? ukuiSwitch.checkedDisableColor : control.pressed ? ukuiSwitch.checkedClickColor : control.hovered ? ukuiSwitch.checkedHoverColor : ukuiSwitch.checkedNormalColor)
                               : (!control.enabled ? ukuiSwitch.disableColor : control.pressed ? ukuiSwitch.clickColor : control.hovered ? ukuiSwitch.hoverColor : ukuiSwitch.normalColor))

        border.color: getStartColor(control.checked ? (!control.enabled ? ukuiSwitch.checkedDisableBorderColor : control.pressed ? ukuiSwitch.checkedClickBorderColor : control.hovered ? ukuiSwitch.checkedHoverBorderColor : ukuiSwitch.checkedNormalBorderColor)
                               : (!control.enabled ? ukuiSwitch.disableBorderColor : control.pressed ? ukuiSwitch.clickBorderColor : control.hovered ? ukuiSwitch.hoverColor : ukuiSwitch.normalBorderColor))

        border.width: ukuiSwitch.borderWidth
        Rectangle {
            x: Math.max(ukuiSwitch.indicatorLeftRightPadding, Math.min(parent.width - width - ukuiSwitch.indicatorLeftRightPadding, control.visualPosition * parent.width - (width / 2) - ukuiSwitch.indicatorLeftRightPadding))
            y: (parent.height - height) / 2
            width: ukuiSwitch.indicatorWidth
            height: ukuiSwitch.indicatorWidth
            radius: height/2
            color: getStartColor(!control.enabled ? ukuiSwitch.disableIndicatorColor : ukuiSwitch.normalIndicatorColor)
//            border.width: control.visualFocus ? 2 : 1
//            border.color: control.visualFocus ? control.palette.highlight : control.enabled ? control.palette.mid : control.palette.midlight

            Behavior on x {
                enabled: !control.down
                SmoothedAnimation { velocity: 200 }
            }
        }
    }

    contentItem: Label {
        leftPadding: control.indicator && !control.mirrored ? control.indicator.width + control.spacing : 0
        rightPadding: control.indicator && control.mirrored ? control.indicator.width + control.spacing : 0

        text: control.text
//        font: control.font
//        color: control.palette.windowText
    }
}
