/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Controls.impl 2.12
import QtQuick.Templates 2.12 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate

T.TextField {
    id: control

    implicitWidth: implicitBackgroundWidth + leftInset + rightInset
                   || Math.max(contentWidth, placeholder.implicitWidth) + leftPadding + rightPadding
    implicitHeight: Math.max(implicitBackgroundHeight + topInset + bottomInset,
                             contentHeight + topPadding + bottomPadding,
                             placeholder.implicitHeight + topPadding + bottomPadding)

    padding: textfield.leftRightPadding
    leftPadding: padding + 4

    color: getStartColor(!control.enabled ? textfield.disableTextColor : textfield.normalTextColor)
    selectionColor: control.palette.highlight
    selectedTextColor: control.palette.highlightedText
    placeholderTextColor: getStartColor(!control.enabled ? textfield.placeHolderDisableTextColor : textfield.placeHolderNormalTextColor)
    verticalAlignment: TextInput.AlignVCenter
    font: app.font
    hoverEnabled: true

    property var backColor: !control.enabled ? textfield.placeHolderDisableBC : control.activeFocus ? textfield.inputNormalBC :
                             click ? textfield.placeHolderClickedBC :
                             control.hovered ? textfield.placeHolderHoveredBC : textfield.normalBC
    property var backBorderColor: !control.enabled ? textfield.disableBorderColor : control.activeFocus ? textfield.focusBorderColor :
                                                                                                                           control.hovered ? textfield.hoverBorderColor : textfield.normalBorderColor

//    property bool hover: false
    property bool click: false

    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }
    PlaceholderText {
        id: placeholder
        x: control.leftPadding
        y: control.topPadding
        width: control.width - (control.leftPadding + control.rightPadding)
        height: control.height - (control.topPadding + control.bottomPadding)

        text: control.placeholderText
        font: control.font
        color: getStartColor(!control.enabled ? textfield.placeHolderDisableTextColor : textfield.placeHolderNormalTextColor)
        verticalAlignment: control.verticalAlignment
        visible: !control.length && !control.preeditText && (!control.activeFocus || control.horizontalAlignment !== Qt.AlignHCenter)
        elide: Text.ElideRight
        renderType: control.renderType
    }

    background: Rectangle {
        id: backrec
        radius: textfield.radius
        implicitWidth: textfield.normalWidth
        implicitHeight: textfield.normalHeight
        border.width: control.activeFocus ? textfield.focusBorderWidth : textfield.normalWidth
        color: gradientRec.visible ? "transparent" : getStartColor(backColor)
        border.color: getStartColor(backBorderColor)

        Rectangle{
            id: gradientRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                GradientStop { position: 0.0; color: getStartColor(backColor) }

                GradientStop { position: 1.0; color: getEndColor(backColor)}
            }
            visible: !parseInterface.isSolidPattern(backColor)
        }
    }

    StylePrivate.UKUITextFiled{
        id: textfield
    }

    StylePrivate.APPParameter{
        id: app
    }

    onPressed: {
        click = true;
    }
    onReleased: click = false
}
