/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.6
import QtGraphicalEffects 1.0
import QtQuick.Controls 2.5 as Controls
import QtQuick.Templates 2.5 as T
import QtQuick.Window 2.15
import org.ukui.qqc2style.private 1.0 as StylePrivate


T.ToolTip {
    id: controlRoot

    palette: appP.palette
    font: appP.font


    x: parent ? Math.round((parent.width - implicitWidth) / 2) : 0
    y: -implicitHeight - 3

    // Always show the tooltip on top of everything else
    z: 999

//    implicitWidth: contentItem.implicitWidth + leftPadding + rightPadding
//    implicitHeight: contentItem.implicitHeight + topPadding + bottomPadding

    margins: tooltip.margins
    padding: tooltip.padding


    // Timeout based on text length, from QTipLabel::restartExpireTimer
    timeout: 4000
    delay: 10

    closePolicy: T.Popup.CloseOnEscape | T.Popup.CloseOnPressOutsideParent | T.Popup.CloseOnReleaseOutsideParent

    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    onVisibleChanged: {
        if(popHandle && parent)
        {
            popHandle.setPosion(windowLabel.mapToGlobal(x, y).x, windowLabel.mapToGlobal(x, y).y)
        }
//        popHandle.visible = visible
//        if(visible && popHandle && !popHandle.visible){
//            popHandle.open()
//        }
//        else if(!visible && popHandle && popHandle.visible)
//            popHandle.close()
    }

    StylePrivate.APPParameter{
        id: appP
    }

    StylePrivate.UKUIToolTip{
        id: tooltip
    }

    background: Item{
    Window {
        id: popupwindow
        width: windowLabel ? windowLabel.width : 0
        height: windowLabel ? windowLabel.height : 0
        visible: controlRoot.visible
        x: controlRoot.x
        y: controlRoot.y
//        blurControl: control
        flags: Qt.ToolTip
        //            margins: tooltip.margins
        //padding: tooltip.padding

        Connections {
            target: popHandle
            function onSizeChanged(width, height){
                rec.implicitWidth = width
                rec.implicitHeight = height
            }

        }

        Rectangle{
            id: rec
            visible: parent.visible
            anchors.fill: parent
            radius: tooltip.radius
            color: getStartColor(tooltip.backColor)
            opacity: appP.menuTransparency

            Label {
                id: windowLabel
                anchors.centerIn: parent
                visible: parent.visible
                text: controlRoot.text
                wrapMode: Text.WordWrap
                font: controlRoot.font
                color: getStartColor(tooltip.textColor)
                onWidthChanged: {
                    controlRoot.implicitWidth = windowLabel.implicitWidth + 2 * controlRoot.padding
                    controlRoot.implicitHeight = windowLabel.implicitHeight+ 2 * controlRoot.padding
                    popHandle.setSize(windowLabel.implicitWidth + 2 * controlRoot.padding, windowLabel.implicitHeight+ 2 * controlRoot.padding)
                }
                onHeightChanged: {
                    controlRoot.implicitWidth = windowLabel.implicitWidth + 2 * controlRoot.padding
                    controlRoot.implicitHeight = windowLabel.implicitHeight+ 2 * controlRoot.padding
                    popHandle.setSize(windowLabel.implicitWidth + 2 * controlRoot.padding, windowLabel.implicitHeight + 2 * controlRoot.padding)
                }
                Component.onCompleted: {
                    controlRoot.implicitWidth = windowLabel.implicitWidth + 2 * controlRoot.padding
                    controlRoot.implicitHeight = windowLabel.implicitHeight+ 2 * controlRoot.padding
                    popHandle.setSize(windowLabel.implicitWidth + 2 * controlRoot.padding, windowLabel.implicitHeight + 2 * controlRoot.padding)
            }
            }
        }
    }
    }


    StylePrivate.PopupHandle{
        id: popHandle
        Component.onCompleted: popHandle.createWindow(popupwindow)
        posFollwMouse: true
    }


    //    contentItem: Controls.Label {
//        text: controlRoot.text
//        wrapMode: Text.WordWrap
//        font: controlRoot.font
//        color: getStartColor(tooltip.textColor)
//        //visible: false
//    }


}
